﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Printing;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using System.Xml.Linq;
using Hitsuji.Nengajou.Models.DataBases;
using Hitsuji.Nengajou.Print.Models.ContentModels;

namespace Hitsuji.Nengajou.Print.Controls
{
    public class PrintIllustLayoutControlClass
    {

        int IllustLayoutCount;
        string textAll = null;
        XElement doc = null;
        IEnumerable<XElement> query;
        //XElement elem;
        XElement innerElem;
        int pageCount;
        int currentPageCount;
        string PrintPaperName = "ﾊｶﾞｷ 100 x 148 mm";

        ComboBox comboBoxFontSize;
        ComboBox comboBoxMargin;
        public void initComboBoxFontSizeItems(ComboBox comboBox)
        {
            this.comboBoxFontSize = comboBox;
            comboBox.Items.Add(4);
            comboBox.Items.Add(2);
            comboBox.Items.Add(0);
            comboBox.Items.Add(-2);
            comboBox.Items.Add(-4);
            comboBox.SelectedItem = -4;
        }

        public void initComboBoxMargineItems(ComboBox comboBox)
        {
            this.comboBoxMargin = comboBox;
            comboBox.Items.Add(10);
            comboBox.Items.Add(5);
            comboBox.Items.Add(0);
            comboBox.Items.Add(-5);
            comboBox.Items.Add(-10);
            comboBox.SelectedItem = -5;
        }


        public void readAndPreviewIllustLayout(object sender, EventArgs e)
        {
            try
            {
                IllustLayoutCount = 0;

                PrintDocument printDocument = new PrintDocument();
                PrintPreviewDialog printPreviewDialog = new PrintPreviewDialog();
                printPreviewDialog.Document = printDocument;

                printDocument.DefaultPageSettings.PaperSize = getPaperSize(printDocument, PrintPaperName);
                
                textAll = File.ReadAllText(FilePathDataBase.getPrintoutIllustLayoutFilePath());
                doc = XElement.Parse(textAll);

                query = from n in doc.Descendants(IllustLayoutInfoTable.PrintDataTableName)
                        //where n.Element("ENABLE").Value.StartsWith(enableParam.ToString())
                        select n;

                //ページ数
                pageCount = query.Count<XElement>();
                currentPageCount = 0;
                if (pageCount >= 1)
                {
                    printDocument.PrintPage += new PrintPageEventHandler(pd_PrintPageIllustLayout);
                }
                //印刷プレビューダイアログを表示する
                printPreviewDialog.ShowDialog();
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.ToString());
                Console.WriteLine(ex.ToString());
            }
        }

        private void pd_PrintPageIllustLayout(object sender, PrintPageEventArgs e)
        {
            try
            {
                var elm = query.ElementAt<XElement>(currentPageCount);
                var illustList = new List<IllustContentModel>();
                var labelList = new List<LabelContentModel>();
                if (elm.Element(IllustLayoutInfoTable.illust) != null)
                {
                    foreach (var el in elm.Elements(IllustLayoutInfoTable.illust))
                    {
                        buildIllustModelInfo(el, illustList);
                    }
                }
                if (elm.HasElements)
                {
                    foreach (var inner in elm.Elements(IllustLayoutInfoTable.label))
                    {
                        innerElem = inner;
                        buildLabelModelInfo(inner, labelList);
                    }
                }


                foreach (var illustModel in illustList)
                {
                    RectangleF rectangleF = new RectangleF(float.Parse(illustModel.X),
                                                           float.Parse(illustModel.Y),
                                                           float.Parse(illustModel.Width) + 100,
                                                           float.Parse(illustModel.Height) + 100);

                    Image img = Image.FromFile(FilePathDataBase.getIllustDirectoryPath() + "\\" + illustModel.Path);
                    var bitmap = new Bitmap(img);
                    e.Graphics.DrawImage(img,
                                         float.Parse(illustModel.X) + (int)this.comboBoxMargin.SelectedItem,
                                         float.Parse(illustModel.Y) + (int)this.comboBoxMargin.SelectedItem,
                                         float.Parse(illustModel.Width),
                                         float.Parse(illustModel.Height));

                }

                foreach (var labelModel in labelList)
                {
                    StringFormat format = new StringFormat();
                    e.Graphics.DrawString(labelModel.Value,
                                            new Font(labelModel.FontFamily, int.Parse(labelModel.FontSize) + (int)this.comboBoxFontSize.SelectedItem),
                                            Hitsuji.Nengajou.Print.Color.ColorExchangeClass.exchangeAsColor(labelModel.FontColor),
                                            float.Parse(labelModel.X) + (int)this.comboBoxMargin.SelectedItem,
                                            float.Parse(labelModel.Y) + (int)this.comboBoxMargin.SelectedItem,
                                            format);
                }

                if (currentPageCount != pageCount - 1)
                {
                    e.HasMorePages = true;
                    currentPageCount += 1;
                }
                else
                {
                    e.HasMorePages = false;
                    currentPageCount = 0;
                }
            }
            catch (Exception ex)
            {
                e.HasMorePages = false;
                System.Windows.Forms.MessageBox.Show("印刷処理時にエラーが発生しました。");
            }

        }

        //用紙サイズ名取得
        public PaperSize getPaperSize(PrintDocument pd, string partOfPaperName)
        {
            for (int index = 0; index < pd.PrinterSettings.PaperSizes.Count; index++)
            {
                if (pd.PrinterSettings.PaperSizes[index].PaperName.Contains(partOfPaperName) == true)
                {
                    return pd.PrinterSettings.PaperSizes[index];
                }
            }
            throw new Exception("用紙サイズ指定時にエラーが発生しました。プリンタ周りの設定を確認してみて下さい。");
        }

        protected void buildLabelModelInfo(XElement elm, List<LabelContentModel> modelList)
        {
            try
            {
                var model = new LabelContentModel();
                model.FontSize = elm.Element(IllustLayoutInfoTable.label_FontSize).Value;
                model.FontFamily = elm.Element(IllustLayoutInfoTable.label_FontFamily).Value;
                model.FontColor = elm.Element(IllustLayoutInfoTable.label_FontColor).Value;
                model.X = elm.Element(IllustLayoutInfoTable.label_X).Value;
                model.Y = elm.Element(IllustLayoutInfoTable.label_Y).Value;
                model.Width = elm.Element(IllustLayoutInfoTable.label_Width).Value;
                model.Height = elm.Element(IllustLayoutInfoTable.label_Height).Value;
                model.Value = elm.Element(IllustLayoutInfoTable.label_Value).Value;
                modelList.Add(model);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }


        protected void buildIllustModelInfo(XElement elm, List<IllustContentModel> modelList)
        {
            try
            {
                var model = new IllustContentModel();
                model.X = elm.Element(IllustLayoutInfoTable.illust_X).Value;
                model.Y = elm.Element(IllustLayoutInfoTable.illust_Y).Value;
                model.Width = elm.Element(IllustLayoutInfoTable.illust_Width).Value;
                model.Height = elm.Element(IllustLayoutInfoTable.illust_Height).Value;
                model.Path = elm.Element(IllustLayoutInfoTable.illust_Image).Value;
                modelList.Add(model);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
    }
}
