﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using System.Xml;
using System.Xml.Linq;
using Hitsuji.Framework.ObserverPattern.Observers;
using Hitsuji.Framework.ObserverPattern.Subjects;
using Hitsuji.Nengajou.Models.Contents;
using Hitsuji.Nengajou.Models.DataBases;
using Hitsuji.Nengajou.ObserverPattern.Subjects;

namespace Hitsuji.Nengajou.ObserverPattern.Observers
{
    public class UpdateIllustLayoutDataSourceObserver : IObserver
    {
        #region IObserver メンバ

        public void doMyRole(ASubject aSubject)
        {
            try
            {
                GetIllustLayoutInfoSubject subject = (GetIllustLayoutInfoSubject)aSubject;
                IllustLayoutInfoContentModel info = subject.models["IllustLayoutInfoContentModel"] as IllustLayoutInfoContentModel;
                IEnumerable<XElement> layoutElements = subject.LayoutElements;

                XDocument doc = new XDocument();
                XElement rootElem = new XElement(IllustLayoutInfoTable.DataSetName);
                int maxId = 0;
                //更新処理分（既存データ）
                foreach (XElement elem in layoutElements)
                {
                    if (string.IsNullOrEmpty(info.Id) ||
                        !info.Id.Equals(elem.Element(IllustLayoutInfoTable.Id).Value))
                    {
                        rootElem.Add(elem);
                        if (maxId < int.Parse(elem.Element(IllustLayoutInfoTable.Id).Value)) maxId 
                            = int.Parse(elem.Element(IllustLayoutInfoTable.Id).Value);
                    }
                    else
                    {
                        //データ更新
                        insertInfoFromContentModel(rootElem, info);
                        if (maxId < int.Parse(info.Id)) maxId = int.Parse(info.Id);
                    }
                }

                //新規登録分
                if (string.IsNullOrEmpty(info.Id))
                {
                    //データ更新
                    //insertInfoFromContentModel(rootElem, info);
                    var layoutElem = new XElement(IllustLayoutInfoTable.DataTableName);
                    layoutElem.Add(new XElement(IllustLayoutInfoTable.Id, maxId + 1));
                    layoutElem.Add(new XElement(IllustLayoutInfoTable.Year, info.LayoutYear));
                    layoutElem.Add(new XElement(IllustLayoutInfoTable.Name, info.LayoutName));
                    foreach (var kvp in info.IllustDictionary)
                    {
                        Image img = kvp.Value;
                        layoutElem.Add(new XElement(IllustLayoutInfoTable.illust,
                                            new XElement(IllustLayoutInfoTable.illust_Name, img.Name),
                                            new XElement(IllustLayoutInfoTable.illust_X, Canvas.GetLeft(img)),
                                            new XElement(IllustLayoutInfoTable.illust_Y, Canvas.GetTop(img)),
                                            new XElement(IllustLayoutInfoTable.illust_Width, img.Width),
                                            new XElement(IllustLayoutInfoTable.illust_Height, img.Height),
                                            new XElement(IllustLayoutInfoTable.illust_Image, 
                                                        (img.Source as BitmapImage).UriSource.LocalPath
                                                                    .Replace(FilePathDataBase.getIllustDirectoryPath() + "\\", ""))
                                            )
                                        );
                    }
                    foreach (var kvp in info.LabelDictionary)
                    {
                        Label label = kvp.Value as Label;
                        layoutElem.Add(new XElement(IllustLayoutInfoTable.label,
                                            new XElement(IllustLayoutInfoTable.label_Name, label.Name),
                                            new XElement(IllustLayoutInfoTable.label_FontSize, label.FontSize.ToString()),
                                            new XElement(IllustLayoutInfoTable.label_FontFamily, label.FontFamily.ToString()),
                                            new XElement(IllustLayoutInfoTable.label_FontColor, label.Foreground.ToString()),
                                            new XElement(IllustLayoutInfoTable.label_X, Canvas.GetLeft(label).ToString()),
                                            new XElement(IllustLayoutInfoTable.label_Y, Canvas.GetTop(label).ToString()),
                                            new XElement(IllustLayoutInfoTable.label_Width, label.Width.ToString()),
                                            new XElement(IllustLayoutInfoTable.label_Height, label.Height.ToString()),
                                            new XElement(IllustLayoutInfoTable.label_Value, label.Content.ToString())
                                            )
                                        );
                    }
                    rootElem.Add(layoutElem);
                }

                doc.Add(rootElem);

                var writer = new XmlTextWriter(FilePathDataBase.getIllustLayoutFilePath(), Encoding.UTF8);
                writer.Formatting = Formatting.Indented;
                doc.WriteTo(writer);
                writer.Close();
                doc = null;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        #endregion

        protected void insertInfoFromContentModel(XElement rootElem, 
                                                  IllustLayoutInfoContentModel info)
        {
            var layoutElem = new XElement(IllustLayoutInfoTable.DataTableName);
            layoutElem.Add(new XElement(IllustLayoutInfoTable.Id, info.Id));
            layoutElem.Add(new XElement(IllustLayoutInfoTable.Year, info.LayoutYear));
            layoutElem.Add(new XElement(IllustLayoutInfoTable.Name, info.LayoutName));
            foreach (var kvp in info.IllustDictionary)
            {
                Image img = kvp.Value;
                layoutElem.Add(new XElement(IllustLayoutInfoTable.illust,
                                    new XElement(IllustLayoutInfoTable.illust_Name, img.Name),
                                    new XElement(IllustLayoutInfoTable.illust_X, Canvas.GetLeft(img)),
                                    new XElement(IllustLayoutInfoTable.illust_Y, Canvas.GetTop(img)),
                                    new XElement(IllustLayoutInfoTable.illust_Width, img.Width),
                                    new XElement(IllustLayoutInfoTable.illust_Height, img.Height),
                                    new XElement(IllustLayoutInfoTable.illust_Image,
                                                (img.Source as BitmapImage).UriSource.LocalPath
                                                            .Replace(FilePathDataBase.getIllustDirectoryPath() + "\\", ""))
                                    )
                                );
            }
            foreach (var kvp in info.LabelDictionary)
            {
                Label label = kvp.Value as Label;
                layoutElem.Add(new XElement(IllustLayoutInfoTable.label,
                                    new XElement(IllustLayoutInfoTable.label_Name, label.Name),
                                    new XElement(IllustLayoutInfoTable.label_FontSize, label.FontSize.ToString()),
                                    new XElement(IllustLayoutInfoTable.label_FontFamily, label.FontFamily.ToString()),
                                    new XElement(IllustLayoutInfoTable.label_FontColor, label.Foreground.ToString()),
                                    new XElement(IllustLayoutInfoTable.label_X, Canvas.GetLeft(label).ToString()),
                                    new XElement(IllustLayoutInfoTable.label_Y, Canvas.GetTop(label).ToString()),
                                    new XElement(IllustLayoutInfoTable.label_Width, label.Width.ToString()),
                                    new XElement(IllustLayoutInfoTable.label_Height, label.Height.ToString()),
                                    new XElement(IllustLayoutInfoTable.label_Value, label.Content.ToString())
                                    )
                                );
            }
            rootElem.Add(layoutElem);
        }

    }
}
