﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Controls;
using System.Xml;
using System.Xml.Linq;
using Hitsuji.Framework.ObserverPattern.Observers;
using Hitsuji.Framework.ObserverPattern.Subjects;
using Hitsuji.Nengajou.Models.Contents;
using Hitsuji.Nengajou.Models.DataBases;
using Hitsuji.Nengajou.ObserverPattern.Subjects;

namespace Hitsuji.Nengajou.ObserverPattern.Observers
{
    public class UpdateReseiverLayoutDataSourceObserver : IObserver
    {
        #region IObserver メンバ

        public void doMyRole(ASubject aSubject)
        {
            try
            {
                GetReseiverLayoutInfoSubject getSubject = (GetReseiverLayoutInfoSubject)aSubject;
                ReseiverLayoutInfoContentModel info = getSubject.models["ReseiverLayoutInfoContentModel"] as ReseiverLayoutInfoContentModel;
                IEnumerable<XElement> layoutElements = getSubject.LayoutElements;
                
                XDocument doc = new XDocument();
                XElement rootElem = new XElement(ReseiverLayoutInfoTable.DataSetName);
                int maxId = 0;
                //更新処理分（既存データ）
                foreach (XElement elem in layoutElements)
                {
                    if (string.IsNullOrEmpty(info.Id) ||
                        !info.Id.Equals(elem.Element(ReseiverLayoutInfoTable.Id).Value))
                    {
                        rootElem.Add(elem);
                        if (maxId < int.Parse(elem.Element(ReseiverLayoutInfoTable.Id).Value)) 
                            maxId = int.Parse(elem.Element(ReseiverLayoutInfoTable.Id).Value);
                    }
                    else
                    {
                        //データ更新
                        insertInfoFromContentModel(rootElem, info);
                        if (maxId < int.Parse(info.Id)) maxId = int.Parse(info.Id);
                    }
                }
                
                //新規登録分
                if (string.IsNullOrEmpty(info.Id))
                {
                    //データ更新
                    //insertInfoFromContentModel(rootElem, info);
                    var layoutElem = new XElement(ReseiverLayoutInfoTable.DataTableName);
                    layoutElem.Add(new XElement(ReseiverLayoutInfoTable.Id, maxId + 1));
                    layoutElem.Add(new XElement(ReseiverLayoutInfoTable.Name, info.LayoutName));
                    foreach (var kvp in info.LabelDictionary)
                    {
                        Label label = kvp.Value as Label;
                        layoutElem.Add(new XElement(ReseiverLayoutInfoTable.label,
                                            new XElement(ReseiverLayoutInfoTable.label_Name, label.Name),
                                            new XElement(ReseiverLayoutInfoTable.label_FontSize, label.FontSize.ToString()),
                                            new XElement(ReseiverLayoutInfoTable.label_FontFamily, label.FontFamily.ToString()),
                                            new XElement(ReseiverLayoutInfoTable.label_FontColor, label.Foreground.ToString()),
                                            new XElement(ReseiverLayoutInfoTable.label_X, Canvas.GetLeft(label).ToString()),
                                            new XElement(ReseiverLayoutInfoTable.label_Y, Canvas.GetTop(label).ToString()),
                                            new XElement(ReseiverLayoutInfoTable.label_Width, label.Width.ToString()),
                                            new XElement(ReseiverLayoutInfoTable.label_Height, label.Height.ToString())
                                            )
                                        );
                    }
                    rootElem.Add(layoutElem);
                }

                doc.Add(rootElem);

                var writer = new XmlTextWriter(FilePathDataBase.getReseiverLayoutFilePath(), Encoding.UTF8);
                writer.Formatting = Formatting.Indented;
                doc.WriteTo(writer);
                writer.Close();
                doc = null;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        #endregion

        protected void insertInfoFromContentModel(XElement rootElem,
                                                  ReseiverLayoutInfoContentModel info)
        {
            var layoutElem = new XElement(ReseiverLayoutInfoTable.DataTableName);
            layoutElem.Add(new XElement(ReseiverLayoutInfoTable.Id, info.Id));
            layoutElem.Add(new XElement(ReseiverLayoutInfoTable.Name, info.LayoutName));
            foreach (var kvp in info.LabelDictionary)
            {
                Label label = kvp.Value as Label;
                layoutElem.Add(new XElement(ReseiverLayoutInfoTable.label,
                                    new XElement(ReseiverLayoutInfoTable.label_Name, label.Name),
                                    new XElement(ReseiverLayoutInfoTable.label_FontSize, label.FontSize.ToString()),
                                    new XElement(ReseiverLayoutInfoTable.label_FontFamily, label.FontFamily.ToString()),
                                    new XElement(ReseiverLayoutInfoTable.label_FontColor, label.Foreground.ToString()),
                                    new XElement(ReseiverLayoutInfoTable.label_X, Canvas.GetLeft(label).ToString()),
                                    new XElement(ReseiverLayoutInfoTable.label_Y, Canvas.GetTop(label).ToString()),
                                    new XElement(ReseiverLayoutInfoTable.label_Width, label.Width.ToString()),
                                    new XElement(ReseiverLayoutInfoTable.label_Height, label.Height.ToString())
                                    )
                                );
            }
            rootElem.Add(layoutElem);
        }

    }
}
