﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Xml.Linq;
using Hitsuji.Framework.ViewModels;
using Hitsuji.Nengajou.Models.Contents;
using Hitsuji.Nengajou.Models.DataBases;
using Hitsuji.Nengajou.Models.InnerCanvasStateModels;
using Hitsuji.Nengajou.ObserverPattern.Observers;
using Hitsuji.Nengajou.ObserverPattern.Subjects;
using Hitsuji.Nengajou.Views;
using Microsoft.Win32;

namespace Hitsuji.Nengajou.ViewModels
{
    public class IllustLayoutViewModel : AViewModel
    {
        public double CanvasWidth
        {
            get { return (this.View as IllustLayoutView).canvas1.Width; }
        }
        public double CanvasHeight
        {
            get { return (this.View as IllustLayoutView).canvas1.Height; }
        }
        //public int BackBordOfCanvasMargine
        //{
        //    get { return 5; }
        //}



        private IllustLayoutInfoContentModel IllustLayoutInfo_;
        public IllustLayoutInfoContentModel IllustLayoutInfo
        {
            get { return IllustLayoutInfo_; }
            set { IllustLayoutInfo_ = value; }
        }

        public IllustLayoutViewModel()
        {
            IllustLayoutInfo = ContentModelFactory.createIllustLayoutContentModel();
            this.IllustLayoutInfo.canExeDelegate_ = this.canExecuteMethod;
            this.IllustLayoutInfo.exeDelegate_ = this.executeMethod;
        }
        

        //ロード
        public void initView()
        {
            if (this.IllustLayoutInfo.LabelDictionary == null)
            {
                this.IllustLayoutInfo.LabelDictionary = new Dictionary<string, Label>();
                this.IllustLayoutInfo.IllustDictionary = new Dictionary<string, Image>();
                initFontSizeComboBox();
                initFontFamilyComboBox();
                initFontColorComboBox();
                
            }
        }

        //フォントサイズコンボボックスの初期設定
        protected void initFontSizeComboBox()
        {
            //TODO:外部データへ
            (this.View as IllustLayoutView).comboBoxFontSize.Items.Clear();
            (this.View as IllustLayoutView).comboBoxFontSize.Items.Add(8);
            (this.View as IllustLayoutView).comboBoxFontSize.Items.Add(10);
            (this.View as IllustLayoutView).comboBoxFontSize.Items.Add(12);
            (this.View as IllustLayoutView).comboBoxFontSize.Items.Add(14);
            (this.View as IllustLayoutView).comboBoxFontSize.Items.Add(16);
            (this.View as IllustLayoutView).comboBoxFontSize.Items.Add(18);
            (this.View as IllustLayoutView).comboBoxFontSize.Items.Add(20);
            (this.View as IllustLayoutView).comboBoxFontSize.Items.Add(24);
        }
        //フォントファミリ
        protected void initFontFamilyComboBox()
        {
            try
            {
                var getSubject = new GetFontFamilyInfoSubject();
                getSubject.doMyRole();
                (this.View as IllustLayoutView).comboBoxFontFamily.Items.Clear();
                foreach (FontFamilyInfoContentModel model in getSubject.models["infoList"] as List<IContentModel>)
                {
                    (this.View as IllustLayoutView).comboBoxFontFamily.Items.Add(model.FontFamilyName);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }       
        }
        //フォントカラーコンボボックスの初期設定
        protected Dictionary<string, Brush> ColorDictionary;
        protected void initFontColorComboBox()
        {
            try
            {
                //TODO:外部データへ
                ColorDictionary = new Dictionary<string, Brush>();
                ColorDictionary.Add("黒色", Brushes.Black);
                ColorDictionary.Add("あお色", Brushes.Blue);
                ColorDictionary.Add("みどり色", Brushes.Green);
                ColorDictionary.Add("赤色", Brushes.Red);
                ColorDictionary.Add("黄色", Brushes.Yellow);
                ColorDictionary.Add("白色", Brushes.White);

                (this.View as IllustLayoutView).comboBoxFontColor.Items.Clear();
                foreach (KeyValuePair<string, Brush> kvp in ColorDictionary)
                {
                    (this.View as IllustLayoutView).comboBoxFontColor.Items.Add(kvp.Key);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        //選択した可変ラベルの各プロパティ値を入力欄に設定
        protected void setPropertiesInInputView(Control control)
        {
            try
            {
                if (control == null) return;
                (this.View as IllustLayoutView).comboBoxFontFamily.SelectedItem = control.FontFamily.ToString();
                (this.View as IllustLayoutView).comboBoxFontSize.SelectedItem = int.Parse(control.FontSize.ToString());
                (this.View as IllustLayoutView).textBoxMultiText.Text = (IllustLayoutInfo.SelectedLabel as Label).Content.ToString();
                foreach (KeyValuePair<string, Brush> kvp in ColorDictionary)
                {
                    if (control.Foreground.ToString().Equals(kvp.Value.ToString()))
                    {
                        (this.View as IllustLayoutView).comboBoxFontColor.SelectedItem = kvp.Key.ToString();
                        break;
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        //選択したイラストの各プロパティ値を入力欄に設定
        protected void setPropertiesInInputView(Image illust)
        {
            if (illust == null) return;

            //自分自身（ラベル）の情報をモデルに登録
            this.IllustLayoutInfo.SelectedIllust = this.IllustLayoutInfo.IllustDictionary[illust.Name];
        }

        //TODO:文字列方向変更時
        //public void onTextVectorChanged(object sender, SelectionChangedEventArgs e)
        //{
        //}
        //フォントサイズ変更時
        public void onFontSizeComboBoxSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (this.IllustLayoutInfo.SelectedLabel != null &&
                (sender as Selector).SelectedItem != null)
            {
                this.IllustLayoutInfo.SelectedLabel.FontSize
                = double.Parse((sender as Selector).SelectedItem.ToString());

            }
        }
        //フォント家コンボボックスオープン時
        public void onFontFamilyComboBoxOpened(object sender, EventArgs e)
        {
            try
            {
                var getSubject = new GetFontFamilyInfoSubject();
                getSubject.doMyRole();
                (this.View as IllustLayoutView).comboBoxFontFamily.Items.Clear();
                foreach (FontFamilyInfoContentModel model in getSubject.models["infoList"] as List<IContentModel>)
                {
                    (this.View as IllustLayoutView).comboBoxFontFamily.Items.Add(model.FontFamilyName);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
        //フォントファミリ変更時
        public void onFontFamilyComboBoxSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (this.IllustLayoutInfo.SelectedLabel != null &&
                (sender as Selector).SelectedItem != null)
            {
                this.IllustLayoutInfo.SelectedLabel.FontFamily
                    = new FontFamily((sender as Selector).SelectedItem.ToString());
            }
        }
        //フォント前景色変更時
        public void onFontColorComboBoxSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (this.IllustLayoutInfo.SelectedLabel != null &&
                (sender as Selector).SelectedItem != null)
            {
                this.IllustLayoutInfo.SelectedLabel.Foreground
                    = ColorDictionary[(sender as Selector).SelectedItem.ToString()] as Brush;
            }
        }
        //マルチテキスト変更
        public void onMultiTextChanged(object sender, TextChangedEventArgs e)
        {
            (IllustLayoutInfo.SelectedLabel as Label).Content = (this.View as IllustLayoutView).textBoxMultiText.Text;
        }
        

        //マウス移動時の処理にしよう
        bool allowDrug_ = false;
        bool allowReSize_ = false;
        Point objectDefaultPosition_;

        //可変ラベルのマウスイベント
        protected void setLabelMouseEvents(Label label)
        {
            //マウスダウン時のイベント登録
            label.PreviewMouseDown += delegate(object obj, MouseButtonEventArgs ev)
            //label.MouseDown += delegate(object obj, MouseButtonEventArgs ev)
            {
                try
                {
                    if (label != null)
                    {
                        Point current = Mouse.GetPosition((this.View as IllustLayoutView).canvas1);
                        objectDefaultPosition_ = ev.GetPosition(label);
                        if ((Canvas.GetLeft(label) + label.Width - 10) < current.X &&
                             current.X < (Canvas.GetLeft(label) + label.Width + 1) ||
                            (Canvas.GetTop(label) + label.Height - 10) < current.Y &&
                             current.Y < (Canvas.GetTop(label) + label.Height + 1))
                        {
                            allowReSize_ = true;
                        }
                        else
                        {
                            allowDrug_ = true;
                        }

                        label.CaptureMouse();

                        //自分自身（ラベル）の情報をモデルに登録
                        this.IllustLayoutInfo.SelectedLabel = this.IllustLayoutInfo.LabelDictionary[label.Name];
                        setPropertiesInInputView(label);
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.ToString());
                }
            };

            //マウス移動時のイベント登録
            label.PreviewMouseMove += delegate(object obj, MouseEventArgs ev)
            //label.MouseMove += delegate(object obj, MouseEventArgs ev)
            {
                try
                {
                    if (allowDrug_)
                    {
                        Point currentMousePosition = Mouse.GetPosition((this.View as IllustLayoutView).canvas1);
                        var control = obj as Label; //obj as UIElement;

                        double xPosition;
                        double yPosition;

                        //xPosition 最小値/最大値を考慮
                        if ((currentMousePosition.X - objectDefaultPosition_.X) < 3)
                        { xPosition = 3; }
                        else if ((currentMousePosition.X - objectDefaultPosition_.X) + control.Width + 3 > CanvasWidth)
                        { xPosition = CanvasWidth - control.Width - 3; }
                        else
                        { xPosition = currentMousePosition.X - objectDefaultPosition_.X; }

                        //yPosition 最小値/最大値を考慮
                        if ((currentMousePosition.Y - objectDefaultPosition_.Y) < 3)
                        { yPosition = 3; }
                        else if ((currentMousePosition.Y - objectDefaultPosition_.Y) + control.Height + 3 > CanvasHeight) // 585
                        { yPosition = CanvasHeight - control.Height - 3; }
                        else
                        { yPosition = currentMousePosition.Y - objectDefaultPosition_.Y; }

                        Canvas.SetLeft(control, xPosition);
                        Canvas.SetTop(control, yPosition);
                    }
                    else if (allowReSize_)
                    {
                        Point currentMousePosition = Mouse.GetPosition((this.View as IllustLayoutView).canvas1);
                        var control = obj as Label; //obj as UIElement;
                        //xPosition 最小値/最大値を考慮
                        if (Canvas.GetLeft(control) + 30 < currentMousePosition.X &&
                            currentMousePosition.X + 3 < CanvasWidth)
                        {
                            control.Width = currentMousePosition.X - Canvas.GetLeft(control) - 3;
                        }
                        if (Canvas.GetTop(control) + 30 < currentMousePosition.Y &&
                            currentMousePosition.Y + 3 < CanvasHeight)
                        {
                            control.Height = currentMousePosition.Y - Canvas.GetTop(control) - 3;
                        }
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.ToString());
                }
            };

            //マウスアップ時のイベント登録
            label.PreviewMouseUp += delegate(object obj, MouseButtonEventArgs ev)
            //label.MouseUp += delegate(object obj, MouseButtonEventArgs ev)
            {
                var control = obj as Label; //obj as UIElement;
                try
                {
                    allowDrug_ = false;
                    allowReSize_ = false;
                    //自分自身（ラベル）の情報をモデルに登録
                    this.IllustLayoutInfo.SelectedLabel = control;
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.ToString());
                }
                control.Background = null;
                control.ReleaseMouseCapture();
            };
        }

        bool illustDrugable_ = false;
        //bool illustReSizeable = false;
        Point illustDefaultPosition_;

        //イラストのマウスイベント
        protected void setIllustMouseEvents(Image illust)
        {
            //マウスダウン時のイベント登録
            illust.PreviewMouseDown += delegate(object obj, MouseButtonEventArgs ev)
            //label.MouseDown += delegate(object obj, MouseButtonEventArgs ev)
            {
                try
                {
                    Image control = obj as Image;
                    if (control != null)
                    {
                        illustDrugable_ = true;
                        illustDefaultPosition_ = ev.GetPosition(control);
                        control.CaptureMouse();

                        //自分自身（イラスト）の情報をモデルに登録
                        this.IllustLayoutInfo.SelectedIllust = this.IllustLayoutInfo.IllustDictionary[control.Name];
                        setPropertiesInInputView(control);
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.ToString());
                }
            };

            //マウス移動時のイベント登録
            illust.PreviewMouseMove += delegate(object obj, MouseEventArgs ev)
            //label.MouseMove += delegate(object obj, MouseEventArgs ev)
            {
                try
                {
                    var control = obj as Image; //obj as UIElement;
                    if (illustDrugable_ == true)
                    {
                        Point currentMousePosition = Mouse.GetPosition((this.View as IllustLayoutView).canvas1);
                        Canvas.SetLeft(control, currentMousePosition.X - illustDefaultPosition_.X);
                        Canvas.SetTop(control, currentMousePosition.Y - illustDefaultPosition_.Y);
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.ToString());
                }
            };

            //マウスアップ時のイベント登録
            illust.PreviewMouseUp += delegate(object obj, MouseButtonEventArgs ev)
            //label.MouseUp += delegate(object obj, MouseButtonEventArgs ev)
            {
                var control = obj as Image;
                try
                {
                    illustDrugable_ = false;
                    //自分自身（ラベル）の情報をモデルに登録
                    this.IllustLayoutInfo.SelectedIllust = control;
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.ToString());
                }
                control.ReleaseMouseCapture();
            };

        }


        //イラスト生成・コレクションへ登録
        public void createAndAddIllust(bool allowDropIllustInCanvas,
                                       string illustName,
                                       string content,
                                       IIllustState state)
        {
            try
            {
                if (allowDropIllustInCanvas)
                {
                    //イラストを生成、コレクションへ登録、キャンバスへ追加
                    var illust = new Image();
                    illust.Name = illustName;
                    illust.Stretch = Stretch.Uniform;
                    illust.Source = new System.Windows.Media.Imaging.BitmapImage(new Uri(FilePathDataBase.getIllustFileFilePath()));
                    setIllustMouseEvents(illust);
                    forceReSizeIllust(illust);
                    (this.View as IllustLayoutView).canvas1.Children.Add(illust);
                    IllustLayoutInfo.IllustDictionary.Add(illust.Name, illust);
                    Canvas.SetTop(illust, 1);
                    Canvas.SetTop(illust, 1);
                    state.setIllustState(illust);
                    //IllustLayoutInfo.SelectedIllust = illust;
                }
                else
                {
                    //イラストをコレクションから削除
                    (this.View as IllustLayoutView).canvas1.Children.Remove(IllustLayoutInfo.IllustDictionary[illustName]);
                    IllustLayoutInfo.IllustDictionary.Remove(illustName);
                    //this.IllustLayoutInfo.SelectedIllust = null;
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        //ラベル生成・コレクションへ登録
        public void createAndAddLabel(bool allowDropLabelInCanvas,
                                       string labelName,
                                       string content,
                                       ILabelState state)
        {
            try
            {
                if (allowDropLabelInCanvas)
                {
                    //ラベルを生成、コレクションへ登録、キャンバスへ追加
                    var label = new Label();
                    label.Name = labelName;
                    label.Content = content;
                    label.FontFamily = new FontFamily("ＭＳ Ｐ明朝");
                    label.FontSize = 14;
                    setLabelMouseEvents(label);
                    (this.View as IllustLayoutView).canvas1.Children.Add(label);
                    IllustLayoutInfo.LabelDictionary.Add(label.Name, label);
                    Canvas.SetTop(label, 1);
                    Canvas.SetTop(label, 1);
                    state.setLabelState(label);
                }
                else
                {
                    //ラベルをコレクションから削除
                    (this.View as IllustLayoutView).canvas1.Children.Remove(IllustLayoutInfo.LabelDictionary[labelName]);
                    IllustLayoutInfo.LabelDictionary.Remove(labelName);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        //イラスト倍かけの制御
        public void multiplicationIllustSize(double param)
        {
            if (IllustLayoutInfo.SelectedIllust == null) return;

            var width = IllustLayoutInfo.SelectedIllust.Width * param;
            var height = IllustLayoutInfo.SelectedIllust.Height * param;
            if ((30 < width) && (30 < height))
            {
                IllustLayoutInfo.SelectedIllust.Width *= param;
                IllustLayoutInfo.SelectedIllust.Height *= param;
            }
        }

        //はがき用紙に対してイラストが大きいときの制御
        public void forceReSizeIllust(Image illust)
        {
            if (illust == null)
                return;

            var w = illust.Source.Width / 390;
            var h = illust.Source.Height / 580;
            if(w > h && w > 1)
            {
                illust.Width = illust.Source.Width / (w + 0.1);
                illust.Height = illust.Source.Height / (w + 0.1);
            }
            else if (h > w && h > 1)
            {
                illust.Width = illust.Source.Width / (h + 0.1);
                illust.Height = illust.Source.Height / (h + 0.1);
            }
            else
            {
                illust.Width = illust.Source.Width;
                illust.Height = illust.Source.Height;
            }
        }

        //年選択コンボ
        public void resetYearSelectionComboBox()
        {
            try
            {
                var getYearDataSubject = new GetYearInfoSubject();
                getYearDataSubject.doMyRole();

                var contentModelList = (ContentModelList)getYearDataSubject.models["infoList"];

                List<string> list = new List<string>();
                foreach (YearInfoContentModel model in contentModelList)
                {
                    list.Add(model.Value);
                }
                (this.View as IllustLayoutView).comboBoxYearSelection.ItemsSource = list;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
        //選択年変更時
        public void onYearSelectionChanged(object sender)
        {
            try
            {
                if ((sender as Selector).SelectedItem == null)
                    return;

                var getIllustLayoutSubject = new GetIllustLayoutInfoSubject();
                var reflectUiIllustLayoutListViewObserver = new ReflectUILayoutListViewAsIllustObserver();
                getIllustLayoutSubject.yearParam = (sender as ComboBox).SelectedItem.ToString();
                getIllustLayoutSubject.addObserver(reflectUiIllustLayoutListViewObserver);
                getIllustLayoutSubject.doMyRole();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        //イラストファイルパスを選択
        public void onClickedButtonIllustPath()
        {
            //カレントディレクトリ取得
            string currentDirectory = System.IO.Directory.GetCurrentDirectory();
            var illust = this.IllustLayoutInfo.SelectedIllust;
            try
            {
                //ダイアログ設定
                var openFileDialog = new OpenFileDialog();
                openFileDialog.InitialDirectory = System.Environment.GetFolderPath(System.Environment.SpecialFolder.Personal);
                openFileDialog.FileName = "jpgファイル(*.jpg)|*.jpg";
                openFileDialog.Title = "jpg, gif, png 形式のファイルを選択してください。";
                openFileDialog.FileName = "";

                if ((bool)openFileDialog.ShowDialog())
                {
                    if (this.IllustLayoutInfo.SelectedIllust == null) return;

                    //選択したイラストをドキュメント配下にコピペして参照するようにする(既存ファイルは上書き)
                    var sourceFile = openFileDialog.FileName;
                    var destFile = FilePathDataBase.getIllustDirectoryPath() + "\\" + openFileDialog.SafeFileName;
                    File.Copy(sourceFile, destFile, true);
                    this.IllustLayoutInfo.SelectedIllust.Source
                        = new System.Windows.Media.Imaging.BitmapImage(new Uri(destFile));
                    //強制的にリサイズ
                    forceReSizeIllust(this.IllustLayoutInfo.SelectedIllust);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
            
            // カレントディレクトリを元に戻す 
            System.IO.Directory.SetCurrentDirectory(currentDirectory);
        }

        //コマンド関連
        public bool canExecuteMethod()
        {
            return !string.IsNullOrEmpty(this.IllustLayoutInfo.LayoutYear) &&
                    !string.IsNullOrEmpty(this.IllustLayoutInfo.LayoutName);
        }

        //コマンド関連
        public void executeMethod()
        {
            if (this.IllustLayoutInfo.LabelDictionary.Count == 0 &&
                this.IllustLayoutInfo.IllustDictionary.Count == 0)
            {
                System.Windows.MessageBox.Show(new StringBuilder("")
                                                    .AppendLine("はがきレイアウトが空のため、登録できません。")
                                                    .AppendLine("ラベル、又はイラストを１つ以上作成してください。").ToString());
                return;
            }

            try
            {
                var getIllustLayoutSubject = new GetIllustLayoutInfoSubject();
                var updateIllustLayoutObserver = new UpdateIllustLayoutDataSourceObserver();
                getIllustLayoutSubject.models.Add(this.IllustLayoutInfo.GetType().Name, this.IllustLayoutInfo);
                getIllustLayoutSubject.addObserver(updateIllustLayoutObserver);
                getIllustLayoutSubject.doMyRole();

                var MainMenuVM = ViewModelManager.getInstance()["MainMenuViewModel"] as MainMenuViewModel;
                MainMenuVM.onIllustLayoutMenuClicked();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }


        //XMLからラベルオブジェクトへ変換
        //キャンバスへの登録など必要な処理を行う
        public void deSerializeXElemAndBuildContents(XElement layoutElement)
        {
            //layoutElementからオブジェクトを生成
            try
            {
                initView();
                this.IllustLayoutInfo.Illust1 = false;
                this.IllustLayoutInfo.Illust2 = false;
                this.IllustLayoutInfo.Illust3 = false;
                this.IllustLayoutInfo.MultiLabel1 = false;
                this.IllustLayoutInfo.MultiLabel2 = false;
                this.IllustLayoutInfo.MultiLabel3 = false;
                this.IllustLayoutInfo.MultiLabel4 = false;

                if (layoutElement == null) return;
                //チェックボックスプロパティを更新（ラベルオブジェクトの生成）
                //各種プロパティを設定
                foreach (var imgElem in layoutElement.Elements("ILLUST"))
                {
                    illustCreator(imgElem);
                }
                foreach (var labelElem in layoutElement.Elements("LBL"))
                {
                    labelCreator(labelElem);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        protected void illustCreator(XElement elem)
        {
            try
            {
                var illustName = elem.Element("NAME").Value;
                if (illustName.Equals("ILLUST1"))
                {
                    IllustLayoutInfo.Illust1 = true;
                }
                else if (illustName.Equals("ILLUST2"))
                {
                    IllustLayoutInfo.Illust2 = true;
                }
                else if (illustName.Equals("ILLUST3"))
                {
                    IllustLayoutInfo.Illust3 = true;
                }

                var illust = this.IllustLayoutInfo.IllustDictionary[illustName];
                Canvas.SetLeft(illust, double.Parse(elem.Element("X").Value));
                Canvas.SetTop(illust, double.Parse(elem.Element("Y").Value));
                illust.Width = double.Parse(elem.Element("WIDTH").Value);
                illust.Height = double.Parse(elem.Element("HEIGHT").Value);
                illust.Source = new BitmapImage(
                                        new Uri(FilePathDataBase.getIllustDirectoryPath() + "\\" + elem.Element("IMG").Value)
                                    );
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        protected void labelCreator(XElement elem)
        {
            try
            {
                //TODO:条件分岐なくす
                var elemName = elem.Element("NAME").Value;
                if (elemName.Equals("LBL_MULTI1"))
                {
                    IllustLayoutInfo.MultiLabel1 = true;
                }
                else if (elemName.Equals("LBL_MULTI2"))
                {
                    IllustLayoutInfo.MultiLabel2 = true;
                }
                else if (elemName.Equals("LBL_MULTI3"))
                {
                    IllustLayoutInfo.MultiLabel3 = true;
                }
                else if (elemName.Equals("LBL_MULTI4"))
                {
                    IllustLayoutInfo.MultiLabel4 = true;
                }

                var label = (this.IllustLayoutInfo.LabelDictionary[elemName]);
                Canvas.SetLeft(label, double.Parse(elem.Element("X").Value));
                Canvas.SetTop(label, double.Parse(elem.Element("Y").Value));
                label.Width = double.Parse(elem.Element("WIDTH").Value);
                label.Height = double.Parse(elem.Element("HEIGHT").Value);
                label.FontSize = double.Parse(elem.Element("F_SIZE").Value);
                label.FontFamily = new FontFamily(elem.Element("F_FAMILY").Value);
                //TODO:リフレクションを利用する
                foreach (var color in ColorDictionary)
                {
                    if (color.Value.ToString().Equals(elem.Element("F_COLOR").Value))
                    {
                        label.Foreground = color.Value;
                        break;
                    }
                }
                label.Content = elem.Element("VALUE").Value;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
    }
}
