﻿using System;
using System.Collections.Generic;
using System.Windows.Controls;
using System.Windows.Media;
using System.Xml.Linq;
using Hitsuji.Framework.ViewModels;
using Hitsuji.Nengajou.Models.Contents;
using Hitsuji.Nengajou.ObserverPattern.Observers;
using Hitsuji.Nengajou.ObserverPattern.Subjects;
using Hitsuji.Nengajou.ViewModels.LayoutListViewItems;
using Hitsuji.Nengajou.Views;
using Hitsuji.Nengajou.Views.LayoutListViewItems;

namespace Hitsuji.Nengajou.ViewModels
{
    public class MainMenuViewModel : AViewModel
    {
        //宛名レイアウト
        public void onReseiverLayoutMenuClicked()
        {
            try
            {
                //プライマリービュー上のオブジェクト変更
                var vmDictionary = ViewModelManager.getInstance();
                var mainAreaBaseVM = vmDictionary["MainAreaBaseViewModel"] as MainAreaBaseViewModel;
                var reseiverLayoutVM = vmDictionary["ReseiverLayoutViewModel"] as ReseiverLayoutViewModel;
                var layoutListVM = vmDictionary["LayoutListViewModel"] as LayoutListViewModel;

                mainAreaBaseVM.clearStackPanelItems();
                mainAreaBaseVM.addStackPanelItem(reseiverLayoutVM.View);
                mainAreaBaseVM.addStackPanelItem(layoutListVM.View);

                (layoutListVM.View as LayoutListView).listBox1.ItemsPanel = null;
                (layoutListVM.View as LayoutListView).listBox1.ItemsPanel
                    = (ItemsPanelTemplate)(layoutListVM.View as LayoutListView).FindResource("LayoutItemsPanelTemplate");

                //リストアイテム選択時の処理を登録
                layoutListVM.OnSelectedItemChangedDelegate = delegate(object selectedItem)
                {
                    try
                    {
                        //選択アイテムのビューとビューモデルを参照
                        var itemV = selectedItem as ReseiverLayoutItemView;
                        var itemVM = itemV.DataContext as LayoutItemViewModel;
                        
                        //レイアウトビューとビューモデルを参照
                        var reseiverLayoutV = reseiverLayoutVM.View as ReseiverLayoutView;

                        //レイアウトビューへ反映(新規登録の場合は終了)
                        if (itemVM == null ||
                            itemVM.LayoutElement == null)
                        {
                            reseiverLayoutVM.ReseiverLayoutInfo.Id = "";
                            reseiverLayoutVM.ReseiverLayoutInfo.LayoutName = "";
                            reseiverLayoutVM.deSerializeXElemAndBuildContents(null);
                            return;
                        }

                        XElement layoutElem = itemVM.LayoutElement;
                        reseiverLayoutVM.ReseiverLayoutInfo.Id = layoutElem.Element("ID").Value;
                        reseiverLayoutVM.ReseiverLayoutInfo.LayoutName = layoutElem.Element("NAME").Value;
                        reseiverLayoutVM.deSerializeXElemAndBuildContents(layoutElem);
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine(ex.ToString());
                    }
                };

                var getReseiverLayoutSubject = new GetReseiverLayoutInfoSubject();
                var reflectUiReseiverLayoutListViewObserver = new ReflectUILayoutListViewAsReseiverObserver();
                getReseiverLayoutSubject.addObserver(reflectUiReseiverLayoutListViewObserver);
                getReseiverLayoutSubject.doMyRole();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        //イラストレイアウト
        public void onIllustLayoutMenuClicked()
        {
            try
            {
                //プライマリービュー上のオブジェクト変更
                var vmDictionary = ViewModelManager.getInstance();
                var mainAreaBaseVM = vmDictionary["MainAreaBaseViewModel"] as MainAreaBaseViewModel;
                var illustLayoutVM = vmDictionary["IllustLayoutViewModel"] as IllustLayoutViewModel;
                var layoutListVM = vmDictionary["LayoutListViewModel"] as LayoutListViewModel;

                mainAreaBaseVM.clearStackPanelItems();
                mainAreaBaseVM.addStackPanelItem(illustLayoutVM.View);
                mainAreaBaseVM.addStackPanelItem(layoutListVM.View);

                (layoutListVM.View as LayoutListView).listBox1.ItemsPanel = null;
                (layoutListVM.View as LayoutListView).listBox1.ItemsPanel
                    = (ItemsPanelTemplate)(layoutListVM.View as LayoutListView).FindResource("LayoutItemsPanelTemplate");

                //リストアイテム選択時の処理を登録
                layoutListVM.OnSelectedItemChangedDelegate = delegate(object selectedItem)
                {
                    try
                    {
                        //選択アイテムのビューとビューモデルを参照
                        var itemV = selectedItem as ReseiverLayoutItemView;
                        var itemVM = itemV.DataContext as LayoutItemViewModel;
                        var illustLayoutV = illustLayoutVM.View as IllustLayoutView;

                        //レイアウトビューへ反映(新規登録の場合は終了)
                        if (itemVM == null ||
                            itemVM.LayoutElement == null)
                        {
                            illustLayoutVM.IllustLayoutInfo.Id = "";
                            illustLayoutVM.IllustLayoutInfo.LayoutName = "";
                            illustLayoutVM.deSerializeXElemAndBuildContents(null);
                            return;
                        }

                        XElement layoutElem = itemVM.LayoutElement;
                        illustLayoutVM.IllustLayoutInfo.Id = layoutElem.Element("ID").Value;
                        illustLayoutVM.IllustLayoutInfo.LayoutName = layoutElem.Element("NAME").Value;
                        illustLayoutVM.deSerializeXElemAndBuildContents(layoutElem);
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine(ex.ToString());
                    }
                };

                (layoutListVM.View as LayoutListView).listBox1.ItemsSource = null;
                (layoutListVM.View as LayoutListView).listBox1.Items.Clear();

                //強制的にクリア（レイアウトリストにアイテムを再表示させるため）
                (illustLayoutVM.View as IllustLayoutView).comboBoxYearSelection.Text = "";
                illustLayoutVM.IllustLayoutInfo.LayoutYear = "";
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }


        //印刷用レイアウト出力
        public void onLayoutAssignMenuClicked()
        {
            try
            {
                var mainMenuV = ViewModelManager.getInstance()["MainMenuViewModel"].View as MainMenuView;

                if (mainMenuV.popupCreatePrintLayout.IsOpen)
                {
                    mainMenuV.popupCreatePrintLayout.IsOpen = false;
                    return;
                }

                mainMenuV.popupCreatePrintLayout.PlacementTarget = mainMenuV.menuItemAssign;
                mainMenuV.popupCreatePrintLayout.Width = 310;
                mainMenuV.popupCreatePrintLayout.Height = 130;

                var getSubject = new GetYearInfoSubject();
                getSubject.doMyRole();
                mainMenuV.CreatePrintLayoutV.comboBoxYearInnerCreateLayoutV.Items.Clear();
                foreach (var yearInfo in getSubject.models["infoList"] as List<IContentModel>)
                {
                    mainMenuV.CreatePrintLayoutV.comboBoxYearInnerCreateLayoutV.Items.Add((yearInfo as YearInfoContentModel).Value);
                }
                mainMenuV.popupCreatePrintLayout.IsOpen = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        //選択されたメニューアイテムの背景色を変更。
        //そのほかのメニューアイテムの背景色は元の色に戻す。
        public void setMenuItemsBackgroundBrush(MenuItem sender)
        {
            foreach (MenuItem item in (this.View as MainMenuView).menu3.Items)
            {
                item.Background = Brushes.Silver;
            }
            sender.Background = Brushes.SlateGray;
        }

    }
}
