﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Linq;
using Hitsuji.Framework.ViewModels;
using Hitsuji.Nengajou.Models.Contents;
using Hitsuji.Nengajou.Models.InnerCanvasStateModels;
using Hitsuji.Nengajou.ObserverPattern.Observers;
using Hitsuji.Nengajou.ObserverPattern.Subjects;
using Hitsuji.Nengajou.Views;

namespace Hitsuji.Nengajou.ViewModels
{
    public class ReseiverLayoutViewModel : AViewModel
    {
        public double CanvasWidth
        {
            get { return (this.View as ReseiverLayoutView).canvas1.Width; }
        }
        public double CanvasHeight
        {
            get { return (this.View as ReseiverLayoutView).canvas1.Height; }
        }
        //public int BackBordOfCanvasMargine
        //{
        //    get { return 5; }
        //}



        private ReseiverLayoutInfoContentModel ReseiverLayoutInfo_;
        public ReseiverLayoutInfoContentModel ReseiverLayoutInfo
        {
            get { return ReseiverLayoutInfo_; }
            set { ReseiverLayoutInfo_ = value; }
        }


        public ReseiverLayoutViewModel()
        {
            ReseiverLayoutInfo = ContentModelFactory.createReseiverLayoutContentModel();
            this.ReseiverLayoutInfo.canExeDelegate_ = this.canExecuteMethod;
            this.ReseiverLayoutInfo.exeDelegate_ = this.executeMethod;
        }

        //ロード
        public void initView()
        {
            if (this.ReseiverLayoutInfo.LabelDictionary == null)
            {
                this.ReseiverLayoutInfo.LabelDictionary = new Dictionary<string, Label>();
                var layoutV = (this.View as ReseiverLayoutView);
                initFontSizeComboBox();
                initFontFamilyComboBox();
                initFontColorComboBox();
            }
        }
        //フォントサイズコンボボックスの初期設定
        protected void initFontSizeComboBox()
        {
            //TODO:外部データへ
            (this.View as ReseiverLayoutView).comboBoxFontSize.Items.Clear();
            (this.View as ReseiverLayoutView).comboBoxFontSize.Items.Add(8);
            (this.View as ReseiverLayoutView).comboBoxFontSize.Items.Add(10);
            (this.View as ReseiverLayoutView).comboBoxFontSize.Items.Add(12);
            (this.View as ReseiverLayoutView).comboBoxFontSize.Items.Add(14);
            (this.View as ReseiverLayoutView).comboBoxFontSize.Items.Add(16);
            (this.View as ReseiverLayoutView).comboBoxFontSize.Items.Add(18);
            (this.View as ReseiverLayoutView).comboBoxFontSize.Items.Add(20);
            (this.View as ReseiverLayoutView).comboBoxFontSize.Items.Add(24);
        }
        //フォントファミリ
        protected void initFontFamilyComboBox()
        {
            try
            {
                var getSubject = new GetFontFamilyInfoSubject();
                getSubject.doMyRole();
                (this.View as ReseiverLayoutView).comboBoxFontFamily.Items.Clear();
                foreach (FontFamilyInfoContentModel model in getSubject.models["infoList"] as List<IContentModel>)
                {
                    (this.View as ReseiverLayoutView).comboBoxFontFamily.Items.Add(model.FontFamilyName);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
        //フォントカラーコンボボックスの初期設定
        protected Dictionary<string, Brush> ColorDictionary;
        protected void initFontColorComboBox()
        {
            try
            {
                //TODO:外部データへ
                ColorDictionary = new Dictionary<string, Brush>();
                ColorDictionary.Add("黒色", Brushes.Black);
                ColorDictionary.Add("あお色", Brushes.Blue);
                ColorDictionary.Add("みどり色", Brushes.Green);
                ColorDictionary.Add("赤色", Brushes.Red);
                ColorDictionary.Add("黄色", Brushes.Yellow);
                //ColorDictionary.Add("白色", Brushes.White);

                (this.View as ReseiverLayoutView).comboBoxFontColor.Items.Clear();
                foreach (KeyValuePair<string, Brush> kvp in ColorDictionary)
                {
                    (this.View as ReseiverLayoutView).comboBoxFontColor.Items.Add(kvp.Key);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        //選択した可変ラベルの各プロパティ値を入力欄に設定
        protected void setPropertiesInInputView(Control control)
        {
            try
            {
                if (control == null) return;
                (this.View as ReseiverLayoutView).comboBoxFontFamily.SelectedItem = control.FontFamily.ToString();
                (this.View as ReseiverLayoutView).comboBoxFontSize.SelectedItem = int.Parse(control.FontSize.ToString());
                foreach (KeyValuePair<string, Brush> kvp in ColorDictionary)
                {
                    if (control.Foreground.ToString().Equals(kvp.Value.ToString()))
                    {
                        (this.View as ReseiverLayoutView).comboBoxFontColor.SelectedItem = kvp.Key.ToString();
                        break;
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        //フォントサイズ変更時
        public void onFontSizeComboBoxSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var reseiverLayoutV = (ReseiverLayoutView)ViewModelManager.getInstance()["ReseiverLayoutViewModel"].View;
            
            if (this.ReseiverLayoutInfo.SelectedLabel != null &&
                (sender as Selector).SelectedItem != null)
            {
                this.ReseiverLayoutInfo.SelectedLabel.FontSize
                    = double.Parse((sender as Selector).SelectedItem.ToString());
            }
        }
        //フォント家コンボボックスオープン時
        public void onFontFamilyComboBoxOpened(object sender, EventArgs e)
        {
            try
            {
                var getSubject = new GetFontFamilyInfoSubject();
                getSubject.doMyRole();
                (this.View as ReseiverLayoutView).comboBoxFontFamily.Items.Clear();
                foreach (FontFamilyInfoContentModel model in getSubject.models["infoList"] as List<IContentModel>)
                {
                    (this.View as ReseiverLayoutView).comboBoxFontFamily.Items.Add(model.FontFamilyName);
                } 
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
        //フォントファミリ変更時
        public void onFontFamilyComboBoxSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var reseiverLayoutV = (ReseiverLayoutView)ViewModelManager.getInstance()["ReseiverLayoutViewModel"].View;

            if (this.ReseiverLayoutInfo.SelectedLabel != null &&
                (sender as Selector).SelectedItem != null)
            {
                this.ReseiverLayoutInfo.SelectedLabel.FontFamily
                    = new FontFamily((sender as Selector).SelectedItem.ToString());
            }
        }
        //フォント前景色変更時
        public void onFontColorComboBoxSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var reseiverLayoutV = (ReseiverLayoutView)ViewModelManager.getInstance()["ReseiverLayoutViewModel"].View;

            if (this.ReseiverLayoutInfo.SelectedLabel != null &&
                (sender as Selector).SelectedItem != null)
            {
                this.ReseiverLayoutInfo.SelectedLabel.Foreground
                    = ColorDictionary[(sender as Selector).SelectedItem.ToString()] as Brush;
            }
        }

        //マウス移動時の処理にしよう
        bool allowDrug_ = false;
        bool allowReSize_ = false;
        Point objectDefaultPosition_;

        //可変ラベルのマウスイベント
        protected void setLabelMouseEvents(Label label)
        {
            //マウスダウン時のイベント登録
            label.PreviewMouseDown += delegate(object obj, MouseButtonEventArgs ev)
            //label.MouseDown += delegate(object obj, MouseButtonEventArgs ev)
            {
                try
                {
                    if (label != null)
                    {
                        Point current = Mouse.GetPosition((this.View as ReseiverLayoutView).canvas1);
                        objectDefaultPosition_ = ev.GetPosition(label);
                        if ((Canvas.GetTop(label) + label.Height - 10) < current.Y &&
                             current.Y < (Canvas.GetTop(label) + label.Height + 1))
                        {
                            allowReSize_ = true;
                        }
                        else
                        {
                            allowDrug_ = true;
                        }

                        label.CaptureMouse();

                        //自分自身（ラベル）の情報をモデルに登録
                        this.ReseiverLayoutInfo.SelectedLabel = this.ReseiverLayoutInfo.LabelDictionary[label.Name];
                        setPropertiesInInputView(obj as Control);
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.ToString());
                }
            };

            //マウス移動時のイベント登録
            label.PreviewMouseMove += delegate(object obj, MouseEventArgs ev)
            //label.MouseMove += delegate(object obj, MouseEventArgs ev)
            {
                try
                {
                    if (allowDrug_ == true)
                    {
                        Point currentMousePosition = Mouse.GetPosition((this.View as ReseiverLayoutView).canvas1);
                        var control = obj as Label; //obj as UIElement;

                        double xPosition;
                        double yPosition;

                        //xPosition 最小値/最大値を考慮
                        if ((currentMousePosition.X - objectDefaultPosition_.X) < 3)
                        { xPosition = 3; }
                        else if ((currentMousePosition.X - objectDefaultPosition_.X) + control.Width + 3 > CanvasWidth)
                        { xPosition = CanvasWidth - control.Width - 3; }
                        else
                        { xPosition = currentMousePosition.X - objectDefaultPosition_.X; }

                        //yPosition 最小値/最大値を考慮
                        if ((currentMousePosition.Y - objectDefaultPosition_.Y) < 3)
                        { yPosition = 3; }
                        else if ((currentMousePosition.Y - objectDefaultPosition_.Y) + control.Height + 3 > CanvasHeight) // 585
                        { yPosition = CanvasHeight - control.Height - 3; }
                        else
                        { yPosition = currentMousePosition.Y - objectDefaultPosition_.Y; }

                        Canvas.SetLeft(control, xPosition);
                        Canvas.SetTop(control, yPosition);
                    }
                    else if (allowReSize_ == true)
                    {
                        Point currentMousePosition = Mouse.GetPosition((this.View as ReseiverLayoutView).canvas1);
                        var control = obj as Label; //obj as UIElement;
                        if (Canvas.GetTop(control) + 30 < currentMousePosition.Y &&
                            currentMousePosition.Y + 3 < CanvasHeight)
                        {
                            control.Height = currentMousePosition.Y - Canvas.GetTop(control) - 3;
                        }
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.ToString());
                }
            };

            //マウスアップ時のイベント登録
            label.PreviewMouseUp += delegate(object obj, MouseButtonEventArgs ev)
            //label.MouseUp += delegate(object obj, MouseButtonEventArgs ev)
            {
                var control = obj as Label; //obj as UIElement;
                try
                {
                    allowDrug_ = false;
                    allowReSize_ = false;
                    //自分自身（ラベル）の情報をモデルに登録
                    this.ReseiverLayoutInfo.SelectedLabel = control;
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.ToString());
                }
                finally
                {
                    control.Background = null;
                    control.ReleaseMouseCapture();
                }
            };
        }

        //ラベル生成・キャンバスへ追加
        public void createAndAddLabel(bool allowAddCanvas,
                                       string labelName, 
                                       string content,
                                       ILabelState state)
        {
            try
            {
                if (allowAddCanvas)
                {
                    //ラベルを生成、コレクションへ登録、キャンバスへ追加
                    var label = new Label();
                    label.Name = labelName;
                    label.Content = content;
                    label.FontFamily = new FontFamily("ＭＳ Ｐ明朝");
                    label.FontSize = 14;
                    setLabelMouseEvents(label);
                    (this.View as ReseiverLayoutView).canvas1.Children.Add(label);
                    ReseiverLayoutInfo.LabelDictionary.Add(label.Name, label);
                    Canvas.SetTop(label, 1);
                    Canvas.SetTop(label, 1);
                    state.setLabelState(label);
                }
                else
                {
                    //ラベルをコレクションから削除
                    (this.View as ReseiverLayoutView).canvas1.Children.Remove(ReseiverLayoutInfo.LabelDictionary[labelName]);
                    ReseiverLayoutInfo.LabelDictionary.Remove(labelName);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }


        //コマンド関連

        public bool canExecuteMethod()
        {
            return !string.IsNullOrEmpty(this.ReseiverLayoutInfo.LayoutName);
        }

        public void executeMethod()
        {
            try
            {
                if (this.ReseiverLayoutInfo.LabelDictionary.Count == 0)
                {
                    System.Windows.MessageBox.Show(new StringBuilder("")
                                                        .AppendLine("はがきレイアウトが空のため、登録できません。")
                                                        .AppendLine("ラベルを１つ以上作成してください。").ToString());
                    return;
                }

                var getReseiverLayoutSubject = new GetReseiverLayoutInfoSubject();
                var updateReseiverLayoutObserver = new UpdateReseiverLayoutDataSourceObserver();
                getReseiverLayoutSubject.models.Add(this.ReseiverLayoutInfo.GetType().Name, this.ReseiverLayoutInfo);
                getReseiverLayoutSubject.addObserver(updateReseiverLayoutObserver);
                getReseiverLayoutSubject.doMyRole();

                var MainMenuVM = ViewModelManager.getInstance()["MainMenuViewModel"] as MainMenuViewModel;
                MainMenuVM.onReseiverLayoutMenuClicked();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        //XMLからラベルオブジェクトへ変換
        //キャンバスへの登録など必要な処理を行う
        public void deSerializeXElemAndBuildContents(XElement layoutElement)
        {
            try
            {
                //layoutElementからオブジェクトを生成
                initView();

                ReseiverLayoutInfo.ReseiverPNo = false;
                ReseiverLayoutInfo.ReseiverAddress1 = false;
                ReseiverLayoutInfo.ReseiverAddress2 = false;
                ReseiverLayoutInfo.ReseiverAddress3 = false;
                ReseiverLayoutInfo.ReseiverName1 = false;
                ReseiverLayoutInfo.ReseiverName2 = false;
                ReseiverLayoutInfo.ReseiverName3 = false;
                ReseiverLayoutInfo.ReseiverName4 = false;
                ReseiverLayoutInfo.ReseiverNamePostFix1 = false;
                ReseiverLayoutInfo.ReseiverNamePostFix2 = false;
                ReseiverLayoutInfo.ReseiverNamePostFix3 = false;
                ReseiverLayoutInfo.ReseiverNamePostFix4 = false;
                ReseiverLayoutInfo.SenderPNo = false;
                ReseiverLayoutInfo.SenderAddress1 = false;
                ReseiverLayoutInfo.SenderAddress2 = false;
                ReseiverLayoutInfo.SenderAddress3 = false;
                ReseiverLayoutInfo.SenderName1 = false;
                ReseiverLayoutInfo.SenderName2 = false;
                ReseiverLayoutInfo.SenderName3 = false;
                ReseiverLayoutInfo.SenderName4 = false;
                ReseiverLayoutInfo.SenderTel = false;

                if (layoutElement == null) return;

                //チェックボックスプロパティを更新（ラベルオブジェクトの生成）
                //各種プロパティを設定
                foreach (var labelElem in layoutElement.Elements("LBL"))
                {
                    labelCreator(labelElem);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }

        protected void labelCreator(XElement elem)
        {
            try
            {
                //TODO:条件分岐なくす
                var elemName = elem.Element("NAME").Value;
                if (elemName.Equals("LBL_R_PRIM1") ||
                    elemName.Equals("LBL_R_PRIM2") ||
                    elemName.Equals("LBL_R_PRIM3") ||
                    elemName.Equals("LBL_R_SECO1") ||
                    elemName.Equals("LBL_R_SECO2") ||
                    elemName.Equals("LBL_R_SECO3") ||
                    elemName.Equals("LBL_R_SECO4"))
                {
                    Label control;
                    if (!ReseiverLayoutInfo.LabelDictionary.TryGetValue(elemName, out control))
                    {
                        ReseiverLayoutInfo.ReseiverPNo = true;
                    }
                }
                else if (elemName.Equals("LBL_R_ADDR1"))
                {
                    ReseiverLayoutInfo.ReseiverAddress1 = true;
                }
                else if (elemName.Equals("LBL_R_ADDR2"))
                {
                    ReseiverLayoutInfo.ReseiverAddress2 = true;
                }
                else if (elemName.Equals("LBL_R_ADDR3"))
                {
                    ReseiverLayoutInfo.ReseiverAddress3 = true;
                }
                else if (elemName.Equals("LBL_R_NAME1"))
                {
                    ReseiverLayoutInfo.ReseiverName1 = true;
                }
                else if (elemName.Equals("LBL_R_NAME2"))
                {
                    ReseiverLayoutInfo.ReseiverName2 = true;
                }
                else if (elemName.Equals("LBL_R_NAME3"))
                {
                    ReseiverLayoutInfo.ReseiverName3 = true;
                }
                else if (elemName.Equals("LBL_R_NAME4"))
                {
                    ReseiverLayoutInfo.ReseiverName4 = true;
                }
                else if (elemName.Equals("LBL_R_NPF1"))
                {
                    ReseiverLayoutInfo.ReseiverNamePostFix1 = true;
                }
                else if (elemName.Equals("LBL_R_NPF2"))
                {
                    ReseiverLayoutInfo.ReseiverNamePostFix2 = true;
                }
                else if (elemName.Equals("LBL_R_NPF3"))
                {
                    ReseiverLayoutInfo.ReseiverNamePostFix3 = true;
                }
                else if (elemName.Equals("LBL_R_NPF4"))
                {
                    ReseiverLayoutInfo.ReseiverNamePostFix4 = true;
                }
                else if (elemName.Equals("LBL_S_PRIM1") ||
                         elemName.Equals("LBL_S_PRIM2") ||
                         elemName.Equals("LBL_S_PRIM3") ||
                         elemName.Equals("LBL_S_SECO1") ||
                         elemName.Equals("LBL_S_SECO2") ||
                         elemName.Equals("LBL_S_SECO3") ||
                         elemName.Equals("LBL_S_SECO4"))
                {
                    Label control;
                    if (!ReseiverLayoutInfo.LabelDictionary.TryGetValue(elemName, out control))
                    {
                        ReseiverLayoutInfo.SenderPNo = true;
                    }
                }
                else if (elemName.Equals("LBL_S_ADDR1"))
                {
                    ReseiverLayoutInfo.SenderAddress1 = true;
                }
                else if (elemName.Equals("LBL_S_ADDR2"))
                {
                    ReseiverLayoutInfo.SenderAddress2 = true;
                }
                else if (elemName.Equals("LBL_S_ADDR3"))
                {
                    ReseiverLayoutInfo.SenderAddress3 = true;
                }
                else if (elemName.Equals("LBL_S_NAME1"))
                {
                    ReseiverLayoutInfo.SenderName1 = true;
                }
                else if (elemName.Equals("LBL_S_NAME2"))
                {
                    ReseiverLayoutInfo.SenderName2 = true;
                }
                else if (elemName.Equals("LBL_S_NAME3"))
                {
                    ReseiverLayoutInfo.SenderName3 = true;
                }
                else if (elemName.Equals("LBL_S_NAME4"))
                {
                    ReseiverLayoutInfo.SenderName4 = true;
                }
                else if (elemName.Equals("LBL_S_TEL"))
                {
                    ReseiverLayoutInfo.SenderTel = true;
                }

                var label = (this.ReseiverLayoutInfo.LabelDictionary[elemName]);
                Canvas.SetLeft(label, double.Parse(elem.Element("X").Value));
                Canvas.SetTop(label, double.Parse(elem.Element("Y").Value));
                label.Width = double.Parse(elem.Element("WIDTH").Value);
                label.Height = double.Parse(elem.Element("HEIGHT").Value);
                label.FontSize = double.Parse(elem.Element("F_SIZE").Value);
                label.FontFamily = new FontFamily(elem.Element("F_FAMILY").Value);
                //TODO:リフレクションを利用する
                foreach (var color in ColorDictionary)
                {
                    if (color.Value.ToString().Equals(elem.Element("F_COLOR").Value))
                    {
                        label.Foreground = color.Value;
                        break;
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
    }
}
